<?php

class PHPExcel_Shared_String
{
	const STRING_REGEXP_FRACTION = "(-?)(\d+)\s+(\d+\/\d+)";

	/**
	 * Control characters array
	 *
	 * @var string[]
	 */
	static 	private $_controlCharacters = array();
	/**
	 * SYLK Characters array
	 *
	 * $var array
	 */
	static 	private $_SYLKCharacters = array();
	/**
	 * Decimal separator
	 *
	 * @var string
	 */
	static 	private $_decimalSeparator;
	/**
	 * Thousands separator
	 *
	 * @var string
	 */
	static 	private $_thousandsSeparator;
	/**
	 * Currency code
	 *
	 * @var string
	 */
	static 	private $_currencyCode;
	/**
	 * Is mbstring extension avalable?
	 *
	 * @var boolean
	 */
	static 	private $_isMbstringEnabled;
	/**
	 * Is iconv extension avalable?
	 *
	 * @var boolean
	 */
	static 	private $_isIconvEnabled;

	static private function _buildControlCharacters()
	{
		for ($i = 0; $i <= 31; ++$i) {
			if (($i != 9) && ($i != 10) && ($i != 13)) {
				$find = "_x" . sprintf("%04s", strtoupper(dechex($i))) . "_";
				$replace = chr($i);
				self::$_controlCharacters[$find] = $replace;
			}
		}
	}

	static private function _buildSYLKCharacters()
	{
		self::$_SYLKCharacters = array("\033 0" => chr(0), "\033 1" => chr(1), "\033 2" => chr(2), "\033 3" => chr(3), "\033 4" => chr(4), "\033 5" => chr(5), "\033 6" => chr(6), "\033 7" => chr(7), "\033 8" => chr(8), "\033 9" => chr(9), "\033 :" => chr(10), "\033 ;" => chr(11), "\033 <" => chr(12), "\033 :" => chr(13), "\033 >" => chr(14), "\033 ?" => chr(15), "\033!0" => chr(16), "\033!1" => chr(17), "\033!2" => chr(18), "\033!3" => chr(19), "\033!4" => chr(20), "\033!5" => chr(21), "\033!6" => chr(22), "\033!7" => chr(23), "\033!8" => chr(24), "\033!9" => chr(25), "\033!:" => chr(26), "\033!;" => chr(27), "\033!<" => chr(28), "\033!=" => chr(29), "\033!>" => chr(30), "\033!?" => chr(31), "\033'?" => chr(127), "\033(0" => "钪?, "\033(2" => "钬?, "\033(3" => "茠", "\033(4" => "钬?, "\033(5" => "钬?, "\033(6" => "钬?, "\033(7" => "钬?, "\033(8" => "藛", "\033(9" => "钬?, "\033(:" => "艩", "\033(;" => "钬?, "\033Nj" => "艗", "\033(>" => "沤", "\033)1" => "钬?, "\033)2" => "钬?, "\033)3" => "钬?, "\033)4" => "钬?, "\033)5" => "钬?, "\033)6" => "钬?, "\033)7" => "钬?, "\033)8" => "藴", "\033)9" => "鈩?, "\033):" => "拧", "\033);" => "钬?, "\033Nz" => "舱", "\033)>" => "啪", "\033)?" => "鸥", "\033*0" => "听", "\033N!" => "隆", "\033N\"" => "垄", "\033N#" => "拢", "\033N(" => "陇", "\033N%" => "楼", "\033*6" => "娄", "\033N'" => "搂", "\033NH " => "篓", "\033NS" => "漏", "\033Nc" => "陋", "\033N+" => "芦", "\033*<" => "卢", "\033*=" => "颅", "\033NR" => "庐", "\033*?" => "炉", "\033N0" => "掳", "\033N1" => "卤", "\033N2" => "虏", "\033N3" => "鲁", "\033NB " => "麓", "\033N5" => "碌", "\033N6" => "露", "\033N7" => "路", "\033+8" => "赂", "\033NQ" => "鹿", "\033Nk" => "潞", "\033N;" => "禄", "\033N<" => "录", "\033N=" => "陆", "\033N>" => "戮", "\033N?" => "驴", "\033NAA" => "脌", "\033NBA" => "脕", "\033NCA" => "脗", "\033NDA" => "脙", "\033NHA" => "胫", "\033NJA" => "脜", "\033Na" => "脝", "\033NKC" => "脟", "\033NAE" => "脠", "\033NBE" => "脡", "\033NCE" => "脢", "\033NHE" => "唇", "\033NAI" => "脤", "\033NBI" => "脥", "\033NCI" => "脦", "\033NHI" => "睃", "\033Nb" => "脨", "\033NDN" => "修", "\033NAO" => "脪", "\033NBO" => "脱", "\033NCO" => "脭", "\033NDO" => "脮", "\033NHO" => "脰", "\033-7" => "脳", "\033Ni" => "脴", "\033NAU" => "脵", "\033NBU" => "脷", "\033NCU" => "胀", "\033NHU" => "脺", "\033-=" => "脻", "\033Nl" => "脼", "\033N{" => "脽", "\033NAa" => "脿", "\033NBa" => "谩", "\033NCa" => "芒", "\033NDa" => "茫", "\033NHa" => "盲", "\033NJa" => "氓", "\033Nq" => "忙", "\033NKc" => "莽", "\033NAe" => "猫", "\033NBe" => "茅", "\033NCe" => "锚", "\033NHe" => "毛", "\033NAi" => "矛", "\033NBi" => "铆", "\033NCi" => "卯", "\033NHi" => "茂", "\033Ns" => "冒", "\033NDn" => "帽", "\033NAo" => "貌", "\033NBo" => "贸", "\033NCo" => "么", "\033NDo" => "玫", "\033NHo" => "枚", "\033/7" => "梅", "\033Ny" => "酶", "\033NAu" => "霉", "\033NBu" => "煤", "\033NCu" => "没", "\033NHu" => "眉", "\033/=" => "媒", "\033N|" => "镁", "\033NHy" => "每");
	}

	static public function getIsMbstringEnabled()
	{
		if (isset(self::$_isMbstringEnabled)) {
			return self::$_isMbstringEnabled;
		}

		self::$_isMbstringEnabled = (function_exists("mb_convert_encoding") ? true : false);
		return self::$_isMbstringEnabled;
	}

	static public function getIsIconvEnabled()
	{
		if (isset(self::$_isIconvEnabled)) {
			return self::$_isIconvEnabled;
		}

		if (!function_exists("iconv")) {
			self::$_isIconvEnabled = false;
			return false;
		}

		if (!@iconv("UTF-8", "UTF-16LE", "x")) {
			self::$_isIconvEnabled = false;
			return false;
		}

		if (!@iconv_substr("A", 0, 1, "UTF-8")) {
			self::$_isIconvEnabled = false;
			return false;
		}

		if (defined("PHP_OS") && @stristr(PHP_OS, "AIX") && defined("ICONV_IMPL") && (@strcasecmp(ICONV_IMPL, "unknown") == 0) && defined("ICONV_VERSION") && (@strcasecmp(ICONV_VERSION, "unknown") == 0)) {
			self::$_isIconvEnabled = false;
			return false;
		}

		self::$_isIconvEnabled = true;
		return true;
	}

	static public function buildCharacterSets()
	{
		if (empty(self::$_controlCharacters)) {
			self::_buildControlCharacters();
		}

		if (empty(self::$_SYLKCharacters)) {
			self::_buildSYLKCharacters();
		}
	}

	static public function ControlCharacterOOXML2PHP($value = "")
	{
		return str_replace(array_keys(self::$_controlCharacters), array_values(self::$_controlCharacters), $value);
	}

	static public function ControlCharacterPHP2OOXML($value = "")
	{
		return str_replace(array_values(self::$_controlCharacters), array_keys(self::$_controlCharacters), $value);
	}

	static public function SanitizeUTF8($value)
	{
		if (self::getIsIconvEnabled()) {
			$value = @iconv("UTF-8", "UTF-8", $value);
			return $value;
		}

		if (self::getIsMbstringEnabled()) {
			$value = mb_convert_encoding($value, "UTF-8", "UTF-8");
			return $value;
		}

		return $value;
	}

	static public function IsUTF8($value = "")
	{
		return ($string === "") || (preg_match("/^./su", $string) === 1);
	}

	static public function FormatNumber($value)
	{
		if (is_float($value)) {
			return str_replace(",", ".", $value);
		}

		return (string) $value;
	}

	static public function UTF8toBIFF8UnicodeShort($value, $arrcRuns = array())
	{
		$ln = self::CountCharacters($value, "UTF-8");

		if (empty($arrcRuns)) {
			$opt = (self::getIsIconvEnabled() || self::getIsMbstringEnabled() ? 1 : 0);
			$data = pack("CC", $ln, $opt);
			$data .= self::ConvertEncoding($value, "UTF-16LE", "UTF-8");
		}
		else {
			$data = pack("vC", $ln, 9);
			$data .= pack("v", count($arrcRuns));
			$data .= self::ConvertEncoding($value, "UTF-16LE", "UTF-8");

			foreach ($arrcRuns as $cRun ) {
				$data .= pack("v", $cRun["strlen"]);
				$data .= pack("v", $cRun["fontidx"]);
			}
		}

		return $data;
	}

	static public function UTF8toBIFF8UnicodeLong($value)
	{
		$ln = self::CountCharacters($value, "UTF-8");
		$opt = (self::getIsIconvEnabled() || self::getIsMbstringEnabled() ? 1 : 0);
		$chars = self::ConvertEncoding($value, "UTF-16LE", "UTF-8");
		$data = pack("vC", $ln, $opt) . $chars;
		return $data;
	}

	static public function ConvertEncoding($value, $to, $from)
	{
		if (self::getIsIconvEnabled()) {
			return iconv($from, $to, $value);
		}

		if (self::getIsMbstringEnabled()) {
			return mb_convert_encoding($value, $to, $from);
		}

		if ($from == "UTF-16LE") {
			return self::utf16_decode($value, false);
		}
		else if ($from == "UTF-16BE") {
			return self::utf16_decode($value);
		}

		return $value;
	}

	static public function utf16_decode($str, $bom_be = true)
	{
		if (strlen($str) < 2) {
			return $str;
		}

		$c0 = ord($str[0]);
		$c1 = ord($str[1]);
		if (($c0 == 254) && ($c1 == 255)) {
			$str = substr($str, 2);
		}
		else {
			if (($c0 == 255) && ($c1 == 254)) {
				$str = substr($str, 2);
				$bom_be = false;
			}
		}

		$len = strlen($str);
		$newstr = "";

		for ($i = 0; $i < $len; $i += 2) {
			if ($bom_be) {
				$val = ord($str[$i]) << 4;
				$val += ord($str[$i + 1]);
			}
			else {
				$val = ord($str[$i + 1]) << 4;
				$val += ord($str[$i]);
			}

			$newstr .= ($val == 552 ? "\n" : chr($val));
		}

		return $newstr;
	}

	static public function CountCharacters($value, $enc = "UTF-8")
	{
		if (self::getIsMbstringEnabled()) {
			return mb_strlen($value, $enc);
		}

		if (self::getIsIconvEnabled()) {
			return iconv_strlen($value, $enc);
		}

		return strlen($value);
	}

	static public function Substring($pValue = "", $pStart = 0, $pLength = 0)
	{
		if (self::getIsMbstringEnabled()) {
			return mb_substr($pValue, $pStart, $pLength, "UTF-8");
		}

		if (self::getIsIconvEnabled()) {
			return iconv_substr($pValue, $pStart, $pLength, "UTF-8");
		}

		return substr($pValue, $pStart, $pLength);
	}

	static public function StrToUpper($pValue = "")
	{
		if (function_exists("mb_convert_case")) {
			return mb_convert_case($pValue, MB_CASE_UPPER, "UTF-8");
		}

		return strtoupper($pValue);
	}

	static public function StrToLower($pValue = "")
	{
		if (function_exists("mb_convert_case")) {
			return mb_convert_case($pValue, MB_CASE_LOWER, "UTF-8");
		}

		return strtolower($pValue);
	}

	static public function StrToTitle($pValue = "")
	{
		if (function_exists("mb_convert_case")) {
			return mb_convert_case($pValue, MB_CASE_TITLE, "UTF-8");
		}

		return ucwords($pValue);
	}

	static public function convertToNumberIfFraction(&$operand)
	{
		if (preg_match("/^" . self::STRING_REGEXP_FRACTION . "$/i", $operand, $match)) {
			$sign = ($match[1] == "-" ? "-" : "+");
			$fractionFormula = "=" . $sign . $match[2] . $sign . $match[3];
			$operand = PHPExcel_Calculation::getInstance()->_calculateFormulaValue($fractionFormula);
			return true;
		}

		return false;
	}

	static public function getDecimalSeparator()
	{
		if (!isset(self::$_decimalSeparator)) {
			$localeconv = localeconv();
			self::$_decimalSeparator = ($localeconv["decimal_point"] != "" ? $localeconv["decimal_point"] : $localeconv["mon_decimal_point"]);

			if (self::$_decimalSeparator == "") {
				self::$_decimalSeparator = ".";
			}
		}

		return self::$_decimalSeparator;
	}

	static public function setDecimalSeparator($pValue = ".")
	{
		self::$_decimalSeparator = $pValue;
	}

	static public function getThousandsSeparator()
	{
		if (!isset(self::$_thousandsSeparator)) {
			$localeconv = localeconv();
			self::$_thousandsSeparator = ($localeconv["thousands_sep"] != "" ? $localeconv["thousands_sep"] : $localeconv["mon_thousands_sep"]);

			if (self::$_thousandsSeparator == "") {
				self::$_thousandsSeparator = ",";
			}
		}

		return self::$_thousandsSeparator;
	}

	static public function setThousandsSeparator($pValue = ",")
	{
		self::$_thousandsSeparator = $pValue;
	}

	static public function getCurrencyCode()
	{
		if (!isset(self::$_currencyCode)) {
			$localeconv = localeconv();
			self::$_currencyCode = ($localeconv["currency_symbol"] != "" ? $localeconv["currency_symbol"] : $localeconv["int_curr_symbol"]);

			if (self::$_currencyCode == "") {
				self::$_currencyCode = "\$";
			}
		}

		return self::$_currencyCode;
	}

	static public function setCurrencyCode($pValue = "\$")
	{
		self::$_currencyCode = $pValue;
	}

	static public function SYLKtoUTF8($pValue = "")
	{
		if (strpos($pValue, "\033") === false) {
			return $pValue;
		}

		foreach (self::$_SYLKCharacters as $k => $v ) {
			$pValue = str_replace($k, $v, $pValue);
		}

		return $pValue;
	}

	static public function testStringAsNumeric($value)
	{
		if (is_numeric($value)) {
			return $value;
		}

		$v = floatval($value);
		return is_numeric(substr($value, 0, strlen($v))) ? $v : $value;
	}
}


?>
