<?php

class PHPExcel_Writer_PDF_Core extends PHPExcel_Writer_HTML
{
	/**
     * Temporary storage directory
     *
     * @var string
     */
	protected $_tempDir = "";
	/**
     * Font
     *
     * @var string
     */
	protected $_font = "freesans";
	/**
     * Orientation (Over-ride)
     *
     * @var string
     */
	protected $_orientation;
	/**
     * Paper size (Over-ride)
     *
     * @var int
     */
	protected $_paperSize;
	/**
     * Temporary storage for Save Array Return type
     *
     * @var string
     */
	private $_saveArrayReturnType;
	/**
     * Paper Sizes xRef List
     *
     * @var array
     */
	static 	protected $_paperSizes = array(
					s => NULL,
					s => NULL,
					s => array(792, 1224),
					s => array(1224, 792),
					s => NULL,
					s => array(396, 612),
					s => NULL,
					s => NULL,
					s => NULL,
					s => NULL,
					s => NULL,
					s => NULL,
					s => NULL,
					s => NULL,
					s => array(609.45, 779.53),
					s => array(720, 1008),
					s => array(792, 1224),
					s => NULL,
					s => array(279, 639),
					s => array(297, 684),
					s => array(324, 747),
					s => array(342, 792),
					s => array(360, 828),
					s => array(1224, 1584),
					s => array(1584, 2448),
					s => array(2448, 3168),
					s => array(311.81, 623.62),
					s => NULL,
					s => NULL,
					s => NULL,
					s => NULL,
					s => array(323.15, 649.13),
					s => NULL,
					s => NULL,
					s => array(498.9, 354.33),
					s => array(311.81, 651.97),
					s => array(279, 540),
					s => array(261, 468),
					s => array(1071, 792),
					s => array(612, 864),
					s => NULL,
					s => NULL,
					s => array(566.93, 419.53),
					s => array(648, 792),
					s => array(720, 792),
					s => array(1080, 792),
					s => array(623.62, 623.62),
					s => array(667.8, 864),
					s => array(667.8, 1080),
					s => array(841.68, 1296),
					s => array(668.98, 912.76),
					s => array(595.8, 792),
					s => NULL,
					s => array(667.8, 864),
					s => array(643.46, 1009.13),
					s => array(864.57, 1380.47),
					s => array(612, 913.68),
					s => array(595.28, 935.43),
					s => NULL,
					s => array(515.91, 728.5),
					s => array(912.76, 1261.42),
					s => array(493.23, 666.14),
					s => array(569.76, 782.36),
					s => NULL,
					s => NULL,
					s => array(912.76, 1261.42)
		);

	public function __construct(PHPExcel $phpExcel)
	{
		parent::__construct($phpExcel);
		$this->setUseInlineCss(true);
		$this->_tempDir = PHPExcel_Shared_File::sys_get_temp_dir();
	}

	public function getFont()
	{
		return $this->_font;
	}

	public function setFont($fontName)
	{
		$this->_font = $fontName;
		return $this;
	}

	public function getPaperSize()
	{
		return $this->_paperSize;
	}

	public function setPaperSize($pValue = PHPExcel_Worksheet_PageSetup::PAPERSIZE_LETTER)
	{
		$this->_paperSize = $pValue;
		return $this;
	}

	public function getOrientation()
	{
		return $this->_orientation;
	}

	public function setOrientation($pValue = PHPExcel_Worksheet_PageSetup::ORIENTATION_DEFAULT)
	{
		$this->_orientation = $pValue;
		return $this;
	}

	public function getTempDir()
	{
		return $this->_tempDir;
	}

	public function setTempDir($pValue = "")
	{
		if (is_dir($pValue)) {
			$this->_tempDir = $pValue;
		}
		else {
			throw new PHPExcel_Writer_Exception("Directory does not exist: $pValue");
		}

		return $this;
	}

	protected function prepareForSave($pFilename = NULL)
	{
		$this->_phpExcel->garbageCollect();
		$this->_saveArrayReturnType = PHPExcel_Calculation::getArrayReturnType();
		PHPExcel_Calculation::setArrayReturnType(PHPExcel_Calculation::RETURN_ARRAY_AS_VALUE);
		$fileHandle = fopen($pFilename, "w");

		if ($fileHandle === false) {
			throw new PHPExcel_Writer_Exception("Could not open file $pFilename for writing.");
		}

		$this->_isPdf = true;
		$this->buildCSS(true);
		return $fileHandle;
	}

	protected function restoreStateAfterSave($fileHandle)
	{
		fclose($fileHandle);
		PHPExcel_Calculation::setArrayReturnType($this->_saveArrayReturnType);
	}
}


?>
