<?php

class OpenidClient extends OpenidCommon
{
	public $openid_url_identity;
	public $URLs = array();
	public $fields = array(
					s => array(),
					s => array()
		);

	public function OpenidClient()
	{
		$this->OpenidCommon();
	}

	public function SetOpenIDServer($a)
	{
		$this->URLs["openid_server"] = $a;
	}

	public function SetTrustRoot($a)
	{
		$this->URLs["trust_root"] = $a;
	}

	public function SetCancelURL($a)
	{
		$this->URLs["cancel"] = $a;
	}

	public function SetApprovedURL($a)
	{
		$this->URLs["approved"] = $a;
	}

	public function SetRequiredFields($a)
	{
		if (is_array($a)) {
			$this->fields["required"] = $a;
		}
		else {
			$this->fields["required"][] = $a;
		}
	}

	public function SetOptionalFields($a)
	{
		if (is_array($a)) {
			$this->fields["optional"] = $a;
		}
		else {
			$this->fields["optional"][] = $a;
		}
	}

	public function SetIdentity($a)
	{
		$this->openid_url_identity = $this->normalize($a);
	}

	public function GetIdentity()
	{
		return $this->openid_url_identity;
	}

	public function splitResponse($response)
	{
		$r = array();
		$response = explode("\n", $response);

		foreach ($response as $line ) {
			$line = trim($line);

			if ($line != "") {
				list($key, $value) = explode(":", $line, 2);
				$r[trim($key)] = trim($value);
			}
		}

		return $r;
	}

	public function OpenID_Standarize($openid_identity = NULL)
	{
		if ($openid_identity === NULL) {
			$openid_identity = $this->openid_url_identity;
		}

		$u = parse_url(strtolower(trim($openid_identity)));
		if (!isset($u["path"]) || ($u["path"] == "/")) {
			$u["path"] = "";
		}

		if (substr($u["path"], -1, 1) == "/") {
			$u["path"] = substr($u["path"], 0, strlen($u["path"]) - 1);
		}

		if (isset($u["query"])) {
			return $u["host"] . $u["path"] . "?" . $u["query"];
		}
		else {
			return $u["host"] . $u["path"];
		}
	}

	public function HTML2OpenIDServer($content)
	{
		$get = array();
		preg_match_all("/<link[^>]*rel=['\"]openid.server['\"][^>]*href=['\"]([^'\"]+)['\"][^>]*\/?>/i", $content, $matches1);
		preg_match_all("/<link[^>]*href='\"([^'\"]+)['\"][^>]*rel=['\"]openid.server['\"][^>]*\/?>/i", $content, $matches2);
		$servers = array_merge($matches1[1], $matches2[1]);
		preg_match_all("/<link[^>]*rel=['\"]openid.delegate['\"][^>]*href=['\"]([^'\"]+)['\"][^>]*\/?>/i", $content, $matches1);
		preg_match_all("/<link[^>]*href=['\"]([^'\"]+)['\"][^>]*rel=['\"]openid.delegate['\"][^>]*\/?>/i", $content, $matches2);
		$delegates = array_merge($matches1[1], $matches2[1]);
		$ret = array($servers, $delegates);
		return $ret;
	}

	public function GetOpenIDServer($check = true)
	{
		if ($check) {
			$response = $this->request($this->openid_url_identity);
			list($servers, $delegates) = $this->HTML2OpenIDServer($response);

			if (count($servers) == 0) {
				$this->raiseError("openid server not found", JIEQI_ERROR_RETURN);
				return false;
			}

			if (isset($delegates[0]) && ($delegates[0] != "")) {
				$this->SetIdentity($delegates[0]);
			}

			$this->SetOpenIDServer($servers[0]);
			return $servers[0];
		}
		else {
			$this->SetOpenIDServer($this->openid_url_identity);
			return $this->openid_url_identity;
		}
	}

	public function Associate()
	{
		$params = array();
		$params["openid.ns"] = $this->ns;
		$params["openid.mode"] = "associate";
		$params["openid.assoc_type"] = $this->assoc_type;
		$params["openid.session_type"] = $this->session_type;
		$params["openid.dh_modulus"] = base64_encode($this->btwocEncode($this->_openid_dh_modulus));
		$params["openid.dh_gen"] = base64_encode($this->btwocEncode($this->_openid_dh_gen));
		$openid_skey_client = $this->make_skey();
		$params["openid.dh_consumer_public"] = base64_encode($this->btwocEncode($this->bcpowmod($this->_openid_dh_gen, $this->_secret_key, $this->_openid_dh_modulus)));
		$response = $this->request($this->URLs["openid_server"], "POST", $params);
		$data = $this->splitResponse($response);

		if ($data["error_code"] == "unsupported-type") {
			return false;
		}
		else {
			$_SESSION["openid_assoc_handle"] = $data["assoc_handle"];
			$_SESSION["dh_server_public"] = $data["dh_server_public"];

			if ($data["session_type"] != "no-encryption") {
				$_SESSION["openid_mac_key"] = $this->dec_mac_key($data["enc_mac_key"], $data["dh_server_public"]);
			}
			else {
				$_SESSION["openid_mac_key"] = base64_decode($data["mac_key"]);
			}

			return array("openid_skey_client" => $_SESSION["openid_skey_client"], "openid_assoc_handle" => $_SESSION["openid_assoc_handle"], "dh_server_public" => $_SESSION["dh_server_public"], "openid_mac_key" => $_SESSION["openid_mac_key"]);
		}
	}

	public function Checkid_Setup()
	{
		$params = array();
		$params["openid.ns"] = $this->ns;
		$params["openid.mode"] = "checkid_setup";
		$params["openid.claimed_id"] = $this->openid_url_identity;
		$params["openid.identity"] = $this->openid_url_identity;

		if (isset($_SESSION["openid_assoc_handle"])) {
			$params["openid.assoc_handle"] = $_SESSION["openid_assoc_handle"];
		}

		$params["openid.return_to"] = $this->URLs["approved"];
		$params["openid.realm"] = $this->URLs["trust_root"];
		if (isset($this->fields["required"]) && (0 < count($this->fields["required"]))) {
			$params["openid.ns.sreg"] = "http://openid.net/extensions/sreg/1.1";
			$params["openid.sreg.required"] = implode(",", $this->fields["required"]);
		}

		if (isset($this->fields["optional"]) && (0 < count($this->fields["optional"]))) {
			$params["openid.ns.sreg"] = "http://openid.net/extensions/sreg/1.1";
			$params["openid.sreg.optional"] = implode(",", $this->fields["optional"]);
		}

		$this->redirect($this->URLs["openid_server"], "GET", $params);
	}

	public function Checkid_Immediate()
	{
		$params = array();
		$params["openid.ns"] = $this->ns;
		$params["openid.mode"] = "checkid_immediate";
		$params["openid.claimed_id"] = $this->openid_url_identity;
		$params["openid.identity"] = $this->openid_url_identity;
		$params["openid.assoc_handle"] = $_SESSION["openid_assoc_handle"];
		$params["openid.return_to"] = $this->URLs["approved"];
		$params["openid.realm"] = $this->URLs["trust_root"];
		if (isset($this->fields["required"]) && (0 < count($this->fields["required"]))) {
			$params["openid.ns.sreg"] = "http://openid.net/extensions/sreg/1.1";
			$params["openid.sreg.required"] = implode(",", $this->fields["required"]);
		}

		if (isset($this->fields["optional"]) && (0 < count($this->fields["optional"]))) {
			$params["openid.ns.sreg"] = "http://openid.net/extensions/sreg/1.1";
			$params["openid.sreg.optional"] = implode(",", $this->fields["optional"]);
		}

		$response = $this->request($this->URLs["openid_server"], "GET", $params);
		$data = $this->splitResponse($response);
	}

	public function Check_Authentication()
	{
		$signfields = explode(",", $_REQUEST["openid_signed"]);
		$params = array();

		foreach ($signfields as $field ) {
			$params["openid." . $field] = $_REQUEST["openid_" . $field];
		}

		$params["openid.mode"] = "check_authentication";
		$params["openid.sig"] = $_REQUEST["openid_sig"];
		$openid_server = $this->GetOpenIDServer();

		if ($openid_server == false) {
			return false;
		}

		$response = $this->request($openid_server, "POST", $params);
		$data = $this->splitResponse($response);

		if ($data["is_valid"] == "true") {
			return true;
		}
		else {
			return false;
		}
	}
}

require_once (dirname(__FILE__) . "/openidcommon.php");

?>
